@@ -40,11 +40,11 @@ class API(api.API, metaclass=abc.ABCMeta):
     def ls_del(self, switch, if_exists=False):
         """Delete logical switch 'switch' and all its ports
 
-        :param switch:   The name or uuid of the switch
-        :type switch:    string or uuid.UUID
-        :type if_exists: If True, don't fail if the switch doesn't exist
-        :type if_exists: boolean
-        :returns:        :class:`Command` with no result
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param if_exists: If True, don't fail if the switch doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
         """
 
     @abc.abstractmethod
@@ -96,6 +96,17 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns: :class:`Command` with RowView result
         """
 
+    @abc.abstractmethod
+    def ls_get_localnet_ports(self, switch, if_exists=False):
+        """Get localnet ports on logical switch 'switch'
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param if_exists: If True, don't fail if the switch doesn't exist
+        :type if_exists:  boolean
+        :returns: :class:`Command` with the RowView list result
+        """
+
     @abc.abstractmethod
     def acl_add(self, switch, direction, priority, match, action, log=False):
         """Add an ACL to 'switch'
@@ -206,6 +217,68 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:          :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def address_set_add(self, name, addresses=None, may_exist=False):
+        """Create a set of addresses named 'name'
+
+        :param name:      The name of the address set
+        :type name:       string
+        :param addresses: One or more IP addresses to add to the address set
+        :type addresses:  list of strings
+        :param may_exist: If True, don't fail if the address set already exists
+        :type may_exist:  boolean
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_del(self, address_set, if_exists=False):
+        """Delete a set of addresses 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param if_exists:   Do not fail if the Address_set row does not exist
+        :type if_exists:    bool
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_get(self, address_set):
+        """Get a set of addresses for 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_list(self):
+        """Get all sets of addresses
+
+        :returns: :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def address_set_add_addresses(self, address_set, addresses):
+        """Add a list of addresses to 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to add to the address set
+        :type addresses:    string or list of strings
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_remove_addresses(self, address_set, addresses):
+        """Remove a list of addresses from 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to remove from address set
+        :type addresses:    string of list of strings
+        :returns:           :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def qos_add(self, switch, direction, priority, match, rate=None,
                 burst=None, dscp=None, may_exist=False, **columns):
@@ -547,6 +620,15 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lrp_get(self, port):
+        """Get logical router port for 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def lrp_set_enabled(self, port, is_enabled):
         """Set administrative state of 'port'
@@ -587,6 +669,70 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:    :class:`Command` with dict result
         """
 
+    @abc.abstractmethod
+    def lrp_set_gateway_chassis(self, port, gateway_chassis, priority=0):
+        """Set gateway chassis for 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param priority:        The priority of the gateway chassis
+        :type priority:         int
+        :returns:               :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_get_gateway_chassis(self, port):
+        """Get the names of all gateway chassis on 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :type port:  int
+        :returns:    :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def lrp_del_gateway_chassis(self, port, gateway_chassis, if_exists=False):
+        """Delete gateway chassis from 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param if_exists:       If True, don't fail if the gateway chassis
+                                doesn't exist
+        :type if_exists:        boolean
+        :returns:               :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_add_networks(self, port, networks, may_exist=False):
+        """Add a network to 'port'
+
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param networks:  One or more IP address/netmask to assign to the port
+        :type networks:   list or string
+        :param may_exist: If True, don't fail if the networks already exists
+        :type may_exist:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_del_networks(self, port, networks, if_exists=False):
+        """Remove a network from 'port'
+
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param networks:  One or more IP address/netmask to remove from
+                          the port
+        :type networks:   list or string
+        :param if_exists: If True, don't fail if the networks doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
@@ -777,6 +923,88 @@ class API(api.API, metaclass=abc.ABCMeta):
     def lb_list(self):
         """Get the UUIDs of all load balanacers"""
 
+    @abc.abstractmethod
+    def lb_get(self, lb):
+        """Get load balancer for 'lb'
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lb_add_health_check(self, lb, vip, **options):
+        """Add health check for 'lb'
+
+        :param lb:      The name or uuid of a load balancer
+        :type lb:       string or uuid.UUID
+        :param vip:     A virtual IP
+        :type vip:      string
+        :param options: keys and values for the port 'options' dict
+        :type options:  key: string, value: string
+        :returns:       :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_del_health_check(self, lb, hc_uuid, if_exists=False):
+        """Remove health check from 'lb'
+
+        :param lb:        The name or uuid of a load balancer
+        :type lb:         string or uuid.UUID
+        :param hc_uuid:   uuid of a health check
+        :type hc_uuid:    uuid.UUID
+        :param if_exists: If True, don't fail if the hc_uuid doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_add_ip_port_mapping(self, lb, endpoint_ip, port_name, source_ip):
+        """Add IP port mapping to 'lb'
+
+        Maps from endpoint IP to a colon-separated pair of logical port name
+        and source IP, e.g. port_name:sourc_ip.
+
+        :param lb:          The name or uuid of a load balancer
+        :type lb:           string or uuid.UUID
+        :param endpoint_ip: IPv4 address
+        :type endpoint_ip:  string
+        :param port_name:   The name of a logical port
+        :type port_name:    string
+        :param source_ip:   IPv4 address
+        :type source_ip:    string
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_del_ip_port_mapping(self, lb, endpoint_ip):
+        """Remove IP port mapping from 'lb'
+
+        :param lb:          The name or uuid of a load balancer
+        :type lb:           string or uuid.UUID
+        :param endpoint_ip: IPv4 address
+        :type endpoint_ip:  string
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def health_check_set_options(self, hc_uuid, **options):
+        """Set options to the 'health_check'
+
+        :param hc_uuid: uuid of the health check
+        :type hc_uuid:  uuid.UUID
+        :param options: keys and values for the port 'options' dict
+        :type options:  key: string, value: string
+        :returns:       :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def health_check_get_options(self, hc_uuid):
+        """Get the options for 'health_check'
+
+        :param hc_uuid: uuid of the health check
+        :type hc_uuid:  uuid.UUID
+        :returns:       :class:`Command` with dict result
+        """
+
     @abc.abstractmethod
     def lr_lb_add(self, router, lb, may_exist=False):
         """Add a load-balancer to 'router'
@@ -1169,3 +1397,72 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type meter:   string or uuid.UUID
         :returns:      :class:`Command` with RowView result
         """
+
+    @abc.abstractmethod
+    def bfd_add(self, logical_port, dst_ip, min_tx=None, min_rx=None,
+                detect_mult=None, external_ids=None, options=None,
+                may_exist=False):
+        """Create a BFD entry
+
+        :param logical_port: Name of logical port where BFD engine should run.
+        :type logical_port:  str
+        :param dst_ip:       BFD peer IP address.
+        :type dst_ip:        str
+        :param min_tx:       This is the minimum interval, in milliseconds,
+                             that the local system would like to use when
+                             transmitting BFD Control packets, less any jitter
+                             applied. The value zero is reserved. Default value
+                             is 1000 ms.
+        :type min_tx:        Optional[int]
+        :param min_rx:       This is the minimum interval, in milliseconds,
+                             between received BFD Control packets that this
+                             system is capable of supporting, less any jitter
+                             applied by the sender. If this value is zero, the
+                             transmitting system does not want the remote
+                             system to send any periodic BFD Control packets.
+        :type min_rx:        Optional[int]
+        :param detect_mult:  Detection time multiplier.  The negotiated
+                             transmit interval, multiplied by this value,
+                             provides the Detection Time for the receiving
+                             system in Asynchronous mode. Default value is 5.
+        :type detect_mult:   Optional[int]
+        :param external_ids: Values to be added as external_id pairs.
+        :type external_ids:  Optional[Dict[str,str]]
+        :param options:      Reserved for future use.
+        :type options:       Optional[Dict[str,str]]
+        :param may_exist:    If True, update any existing BFD entry if it
+                             already exists.  Default is False which will raise
+                             an error if a BFD entry with same logical_port,
+                             dst_ip pair already exists.
+        :type may_exist:     Optional[bool]
+        :returns:            :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def bfd_del(self, uuid):
+        """Delete a BFD entry
+
+        :param uuid:   The uuid of the BFD entry
+        :type uuid:    uuid.UUID
+        :returns:      :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def bfd_find(self, logical_port, dst_ip):
+        """Find BFD entry.
+
+        :param logical_port: Name of logical port where BFD engine runs.
+        :type logical_port:  str
+        :param dst_ip:       BFD peer IP address.
+        :type dst_ip:        str
+        :returns:            :class:`Command` with List[Dict[str,any]] result
+        """
+
+    @abc.abstractmethod
+    def bfd_get(self, uuid):
+        """Get the BFD entry
+
+        :param uuid:   The uuid of the BFD entry
+        :type uuid:    uuid.UUID
+        :returns:      :class:`Command` with RowView result
+        """
