@@ -80,6 +80,28 @@ class LsGetCommand(cmd.BaseGetRowCommand):
     table = 'Logical_Switch'
 
 
+class LSGetLocalnetPortsCommand(cmd.ReadOnlyCommand):
+    def __init__(self, api, switch, if_exists=False):
+        super().__init__(api)
+        self.switch = switch
+        self.if_exists = if_exists
+
+    def localnet_port(self, row):
+        return row.type == const.LOCALNET
+
+    def run_idl(self, txn):
+        try:
+            lswitch = self.api.lookup('Logical_Switch', self.switch)
+            self.result = [rowview.RowView(p) for p in lswitch.ports
+                           if self.localnet_port(p)]
+        except idlutils.RowNotFound as e:
+            if self.if_exists:
+                self.result = []
+                return
+            msg = "Logical Switch %s does not exist" % self.switch
+            raise RuntimeError(msg) from e
+
+
 class _AclAddHelper(cmd.AddCommand):
     table_name = 'ACL'
 
@@ -300,7 +322,8 @@ class QoSAddCommand(cmd.AddCommand):
     table_name = 'QoS'
 
     def __init__(self, api, switch, direction, priority, match, rate=None,
-                 burst=None, dscp=None, may_exist=False, **columns):
+                 burst=None, dscp=None, external_ids_match=None,
+                 may_exist=False, **columns):
         if direction not in ('from-lport', 'to-lport'):
             raise TypeError("direction must be either from-lport or to-lport")
         if not 0 <= priority <= const.ACL_PRIORITY_MAX:
@@ -317,6 +340,9 @@ class QoSAddCommand(cmd.AddCommand):
                              dscp, const.QOS_DSCP_MAX)
         if rate is None and dscp is None:
             raise ValueError("One of the rate or dscp must be configured")
+        if (external_ids_match is not None and
+                not isinstance(external_ids_match, dict)):
+            raise ValueError("external_ids_match must be None or a dictionary")
         super().__init__(api)
         self.switch = switch
         self.direction = direction
@@ -327,8 +353,12 @@ class QoSAddCommand(cmd.AddCommand):
         self.dscp = dscp
         self.may_exist = may_exist
         self.columns = columns
+        self.external_ids_match = external_ids_match
 
     def qos_match(self, row):
+        if self.external_ids_match:
+            return self.external_ids_match.items() <= row.external_ids.items()
+
         return (self.direction == row.direction and
                 self.priority == row.priority and
                 self.match == row.match)
@@ -337,10 +367,13 @@ class QoSAddCommand(cmd.AddCommand):
         ls = self.api.lookup('Logical_Switch', self.switch)
         for qos_rule in iter(r for r in ls.qos_rules if self.qos_match(r)):
             if self.may_exist:
-                self.result = rowview.RowView(qos_rule)
-                return
+                return self._update_qos(qos_rule)
             raise RuntimeError("QoS (%s, %s, %s) already exists" % (
                 self.direction, self.priority, self.match))
+
+        self._create_qos(txn, ls)
+
+    def _create_qos(self, txn, ls):
         row = txn.insert(self.api.tables[self.table_name])
         row.direction = self.direction
         row.priority = self.priority
@@ -355,6 +388,30 @@ class QoSAddCommand(cmd.AddCommand):
         ls.addvalue('qos_rules', row)
         self.result = row.uuid
 
+    def _update_qos(self, qos_rule):
+        qos_rule.direction = self.direction
+        qos_rule.priority = self.priority
+        qos_rule.match = self.match
+        if self.rate:
+            qos_rule.setkey('bandwidth', 'rate', self.rate)
+            if self.burst:
+                qos_rule.setkey('bandwidth', 'burst', self.burst)
+            else:
+                qos_rule.delkey('bandwidth', 'burst')
+        else:
+            qos_rule.delkey('bandwidth', 'rate')
+            qos_rule.delkey('bandwidth', 'burst')
+
+        if self.dscp:
+            qos_rule.setkey('action', 'dscp', self.dscp)
+        else:
+            qos_rule.delkey('action', 'dscp')
+
+        for column, value in self.columns.items():
+            if getattr(qos_rule, column) != value:
+                self.set_column(qos_rule, column, value)
+        self.result = qos_rule.uuid
+
 
 class QoSDelCommand(cmd.BaseCommand):
     def __init__(self, api, switch, direction=None,
@@ -410,8 +467,7 @@ class QoSDelExtIdCommand(cmd.BaseCommand):
 
     def run_idl(self, txn):
         try:
-            lswitch = idlutils.row_by_value(self.api.idl, 'Logical_Switch',
-                                            'name', self.lswitch)
+            lswitch = self.api.lookup('Logical_Switch', self.lswitch)
         except idlutils.RowNotFound as e:
             if self.if_exists:
                 return
@@ -661,30 +717,13 @@ class LspGetTypeCommand(cmd.ReadOnlyCommand):
         self.result = lsp.type
 
 
-class LspSetOptionsCommand(cmd.BaseCommand):
+class LspSetOptionsCommand(cmd.BaseSetOptionsCommand):
     table = 'Logical_Switch_Port'
 
-    def __init__(self, api, port, **options):
-        super().__init__(api)
-        self.port = port
-        self.options = options
 
-    def run_idl(self, txn):
-        lsp = self.api.lookup(self.table, self.port)
-        lsp.options = self.options
-
-
-class LspGetOptionsCommand(cmd.ReadOnlyCommand):
+class LspGetOptionsCommand(cmd.BaseGetOptionsCommand):
     table = 'Logical_Switch_Port'
 
-    def __init__(self, api, port):
-        super().__init__(api)
-        self.port = port
-
-    def run_idl(self, txn):
-        lsp = self.api.lookup(self.table, self.port)
-        self.result = lsp.options
-
 
 class LspSetDhcpV4OptionsCommand(cmd.BaseCommand):
     def __init__(self, api, port, dhcpopt_uuid):
@@ -744,25 +783,12 @@ class DhcpOptionsGetCommand(cmd.BaseGetRowCommand):
     table = 'DHCP_Options'
 
 
-class DhcpOptionsSetOptionsCommand(cmd.BaseCommand):
-    def __init__(self, api, dhcpopt_uuid, **options):
-        super().__init__(api)
-        self.dhcpopt_uuid = dhcpopt_uuid
-        self.options = options
-
-    def run_idl(self, txn):
-        dhcpopt = self.api.lookup('DHCP_Options', self.dhcpopt_uuid)
-        dhcpopt.options = self.options
+class DhcpOptionsSetOptionsCommand(cmd.BaseSetOptionsCommand):
+    table = 'DHCP_Options'
 
 
-class DhcpOptionsGetOptionsCommand(cmd.ReadOnlyCommand):
-    def __init__(self, api, dhcpopt_uuid):
-        super().__init__(api)
-        self.dhcpopt_uuid = dhcpopt_uuid
-
-    def run_idl(self, txn):
-        dhcpopt = self.api.lookup('DHCP_Options', self.dhcpopt_uuid)
-        self.result = dhcpopt.options
+class DhcpOptionsGetOptionsCommand(cmd.BaseGetOptionsCommand):
+    table = 'DHCP_Options'
 
 
 class LrAddCommand(cmd.BaseCommand):
@@ -923,11 +949,11 @@ class LrpGetEnabledCommand(cmd.ReadOnlyCommand):
         self.result = next(iter(lrp.enabled), True)
 
 
-class LrpSetOptionsCommand(LspSetOptionsCommand):
+class LrpSetOptionsCommand(cmd.BaseSetOptionsCommand):
     table = 'Logical_Router_Port'
 
 
-class LrpGetOptionsCommand(LspGetOptionsCommand):
+class LrpGetOptionsCommand(cmd.BaseGetOptionsCommand):
     table = 'Logical_Router_Port'
 
 
@@ -950,7 +976,13 @@ class LrpSetGatewayChassisCommand(cmd.BaseCommand):
         lrp.addvalue('gateway_chassis', cmd.result)
 
 
-class LrpGetGatewayChassisCommand(LrpGetOptionsCommand):
+class LrpGetGatewayChassisCommand(cmd.ReadOnlyCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port):
+        super().__init__(api)
+        self.port = port
+
     def run_idl(self, txn):
         lrp = self.api.lookup(self.table, self.port)
         self.result = [rowview.RowView(d) for d in lrp.gateway_chassis]
@@ -978,6 +1010,133 @@ class LrpDelGatewayChassisCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
+class _LrpNetworksCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, networks, exists):
+        super().__init__(api)
+        self.port = port
+        self.exists = exists
+        if isinstance(networks, (str, bytes)):
+            networks = [networks]
+        self.networks = [str(netaddr.IPNetwork(network))
+                         for network in networks]
+
+
+class LrpAddNetworksCommand(_LrpNetworksCommand):
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for network in self.networks:
+            if network in lrp.networks and not self.exists:
+                msg = "Network '%s' already exist in networks of port %s" % (
+                    network, lrp.uuid)
+                raise RuntimeError(msg)
+            lrp.addvalue('networks', network)
+
+
+class LrpDelNetworksCommand(_LrpNetworksCommand):
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for network in self.networks:
+            if network not in lrp.networks and not self.exists:
+                msg = "Network '%s' does not exist in networks of port %s" % (
+                    network, lrp.uuid)
+                raise RuntimeError(msg)
+            lrp.delvalue('networks', network)
+
+
+class BFDFindCommand(cmd.DbFindCommand):
+    table = 'BFD'
+
+    def __init__(self, api, port, dst_ip):
+        super().__init__(
+            api,
+            self.table,
+            ('logical_port', '=', port),
+            ('dst_ip', '=', dst_ip),
+            row=True,
+        )
+
+
+class BFDAddCommand(cmd.AddCommand):
+    # cmd.AddCommand uses self.table_name, other base commands use self.table
+    table_name = 'BFD'
+
+    def __init__(self, api, logical_port, dst_ip, min_tx=None, min_rx=None,
+                 detect_mult=None, external_ids=None, options=None,
+                 may_exist=False):
+        for attr in ('logical_port', 'dst_ip'):
+            if not isinstance(locals().get(attr), str):
+                raise ValueError("%s must be of type str" % attr)
+        for attr in ('min_tx', 'min_rx', 'detect_mult'):
+            value = locals().get(attr)
+            if value and (not isinstance(value, int) or value < 1):
+                raise ValueError("%s must be of type int and > 0" % attr)
+        for attr in ('external_ids', 'options'):
+            value = locals().get(attr)
+            if value and not isinstance(value, dict):
+                raise ValueError("%s must be of type dict" % attr)
+        super().__init__(api)
+        self.logical_port = logical_port
+        self.dst_ip = dst_ip
+        self.columns = {
+            'logical_port': logical_port,
+            'dst_ip': dst_ip,
+            'min_tx': [min_tx] if min_tx else [],
+            'min_rx': [min_rx] if min_rx else [],
+            'detect_mult': [detect_mult] if detect_mult else [],
+            'external_ids': external_ids or {},
+            'options': options or {},
+        }
+        self.may_exist = may_exist
+
+    def run_idl(self, txn):
+        cmd = BFDFindCommand(self.api, self.logical_port, self.dst_ip)
+        cmd.run_idl(txn)
+        bfd_result = cmd.result
+        if bfd_result:
+            if len(bfd_result) > 1:
+                # With the current database schema, this cannot happen, but
+                # better safe than sorry.
+                raise RuntimeError(
+                    "Unexpected duplicates in database for port %s "
+                    "and dst_ip %s" % (self.logical_port, self.dst_ip))
+            bfd = bfd_result[0]
+            if self.may_exist:
+                self.set_columns(bfd, **self.columns)
+                # When no changes are made to a record, the parent
+                # `post_commit` method will not be called.
+                #
+                # Ensure consistent return to caller of `Command.execute()`
+                # even when no changes have been applied.
+                self.result = rowview.RowView(bfd)
+                return
+            else:
+                raise RuntimeError(
+                    "BFD entry for port %s and dst_ip %s exists" % (
+                        self.logical_port, self.dst_ip))
+        bfd = txn.insert(self.api.tables[self.table_name])
+        bfd.logical_port = self.logical_port
+        bfd.dst_ip = self.dst_ip
+        self.set_columns(bfd, **self.columns)
+        # Setting the result to something other than a :class:`rowview.RowView`
+        # or :class:`ovs.db.idl.Row` typed value will make the parent
+        # `post_commit` method retrieve the newly insterted row from IDL and
+        # return that to the caller.
+        self.result = bfd.uuid
+
+
+class BFDDelCommand(cmd.DbDestroyCommand):
+    table = 'BFD'
+
+    def __init__(self, api, uuid):
+        super().__init__(api, self.table, uuid)
+
+
+class BFDGetCommand(cmd.BaseGetRowCommand):
+    table = 'BFD'
+
+
 class LrRouteAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, prefix, nexthop, port=None,
                  policy='dst-ip', may_exist=False):
@@ -1331,6 +1490,100 @@ class LbListCommand(cmd.ReadOnlyCommand):
                        for r in self.api.tables['Load_Balancer'].rows.values()]
 
 
+class LbGetCommand(cmd.BaseGetRowCommand):
+    table = 'Load_Balancer'
+
+
+class LbAddHealthCheckCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, vip, **options):
+        super().__init__(api)
+        self.lb = lb
+        self.vip = vip
+        self.options = options
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        cmd = HealthCheckAddCommand(self.api, self.vip, **self.options)
+        cmd.run_idl(txn)
+        lb.addvalue('health_check', cmd.result)
+
+
+class LbDelHealthCheckCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, hc_uuid, if_exists=False):
+        super().__init__(api)
+        self.lb = lb
+        self.hc_uuid = hc_uuid
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        for health_check in lb.health_check:
+            if health_check.uuid == self.hc_uuid:
+                lb.delvalue('health_check', health_check)
+                health_check.delete()
+                return
+        if not self.if_exists:
+            msg = "Health check '%s' on lb %s does not exist" % (
+                self.hc_uuid, self.lb)
+            raise RuntimeError(msg)
+
+
+class LbAddIpPortMappingCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, endpoint_ip, port_name, source_ip):
+        super().__init__(api)
+        self.lb = lb
+        self.endpoint_ip = str(netaddr.IPAddress(endpoint_ip))
+        self.port_name = port_name
+        self.source_ip = str(netaddr.IPAddress(source_ip))
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        lb.setkey('ip_port_mappings', self.endpoint_ip,
+                  '%s:%s' % (self.port_name, self.source_ip))
+
+
+class LbDelIpPortMappingCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, endpoint_ip):
+        super().__init__(api)
+        self.lb = lb
+        self.endpoint_ip = str(netaddr.IPAddress(endpoint_ip))
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        lb.delkey('ip_port_mappings', self.endpoint_ip)
+
+
+class HealthCheckAddCommand(cmd.AddCommand):
+    table_name = 'Load_Balancer_Health_Check'
+
+    def __init__(self, api, vip, **options):
+        super().__init__(api)
+        self.vip = utils.normalize_ip_port(vip)
+        self.options = options
+
+    def run_idl(self, txn):
+        hc = txn.insert(self.api.tables[self.table_name])
+        hc.vip = self.vip
+        hc.options = self.options
+        self.result = hc.uuid
+
+
+class HealthCheckSetOptionsCommand(cmd.BaseSetOptionsCommand):
+    table = 'Load_Balancer_Health_Check'
+
+
+class HealthCheckGetOptionsCommand(cmd.BaseGetOptionsCommand):
+    table = 'Load_Balancer_Health_Check'
+
+
 class LrLbAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, lb, may_exist=False):
         super().__init__(api)
@@ -1610,7 +1863,7 @@ class GatewayChassisAddCommand(cmd.AddCommand):
         gwc.chassis_name = self.chassis_name
         gwc.priority = self.priority
         self.set_columns(gwc, **self.columns)
-        self.result = gwc
+        self.result = gwc.uuid
 
 
 class HAChassisGroupAddCommand(cmd.AddCommand):
@@ -1665,16 +1918,15 @@ class HAChassisGroupAddChassisCommand(cmd.AddCommand):
 
     def __init__(self, api, hcg_id, chassis, priority, **columns):
         super().__init__(api)
-        self.hcg_id = hcg_id
+        self.hcg = hcg_id
         self.chassis = chassis
         self.priority = priority
         self.columns = columns
 
     def run_idl(self, txn):
-        hc_group = self.api.lookup('HA_Chassis_Group', self.hcg_id)
+        hc_group = self.api.lookup('HA_Chassis_Group', self.hcg)
         found = False
-        hc = None
-        for hc in hc_group.ha_chassis:
+        for hc in getattr(hc_group, 'ha_chassis', []):
             if hc.chassis_name != self.chassis:
                 continue
             found = True
@@ -1696,26 +1948,29 @@ class HAChassisGroupDelChassisCommand(cmd.BaseCommand):
 
     def __init__(self, api, hcg_id, chassis, if_exists=False):
         super().__init__(api)
-        self.hcg_id = hcg_id
+        self.hcg = hcg_id
         self.chassis = chassis
         self.if_exists = if_exists
 
     def run_idl(self, txn):
         try:
-            hc_group = self.api.lookup('HA_Chassis_Group', self.hcg_id)
-        except idlutils.RowNotFound:
+            hc_group = self.api.lookup('HA_Chassis_Group', self.hcg)
+        except idlutils.RowNotFound as exc:
             if self.if_exists:
                 return
 
+            raise RuntimeError('HA Chassis Group %s does not exist' %
+                               utils.get_uuid(self.hcg)) from exc
+
         hc = None
-        for hc in hc_group.ha_chassis:
+        for hc in getattr(hc_group, 'ha_chassis', []):
             if hc.chassis_name == self.chassis:
                 break
         else:
             if self.if_exists:
                 return
             raise RuntimeError(
-                'HA Chassis %s does not exist' % self.hcg_id)
+                'HA Chassis %s does not exist' % utils.get_uuid(self.hcg))
 
         hc_group.delvalue('ha_chassis', hc)
         hc.delete()
