@@ -53,6 +53,9 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
         return self.db_remove('Logical_Switch', switch_uuid, 'dns_records',
                               dns_uuid, if_exists=if_exists)
 
+    def ls_get_localnet_ports(self, switch, if_exists=False):
+        return cmd.LSGetLocalnetPortsCommand(self, switch, if_exists)
+
     def acl_add(self, switch, direction, priority, match, action, log=False,
                 may_exist=False, **external_ids):
         return cmd.AclAddCommand(self, switch, direction, priority,
@@ -99,9 +102,11 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
         return cmd.AddressSetRemoveAddressCommand(self, address_set, addresses)
 
     def qos_add(self, switch, direction, priority, match, rate=None,
-                burst=None, dscp=None, may_exist=False, **columns):
+                burst=None, dscp=None, external_ids_match=None,
+                may_exist=False, **columns):
         return cmd.QoSAddCommand(self, switch, direction, priority, match,
-                                 rate, burst, dscp, may_exist, **columns)
+                                 rate, burst, dscp, external_ids_match,
+                                 may_exist, **columns)
 
     def qos_del(self, switch, direction=None, priority=None, match=None,
                 if_exists=True):
@@ -224,6 +229,12 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
         return cmd.LrpDelGatewayChassisCommand(self, port,
                                                gateway_chassis, if_exists)
 
+    def lrp_add_networks(self, port, networks, may_exist=False):
+        return cmd.LrpAddNetworksCommand(self, port, networks, may_exist)
+
+    def lrp_del_networks(self, port, networks, if_exists=False):
+        return cmd.LrpDelNetworksCommand(self, port, networks, if_exists)
+
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
         return cmd.LrRouteAddCommand(self, router, prefix, nexthop, port,
@@ -271,6 +282,28 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lb_list(self):
         return cmd.LbListCommand(self)
 
+    def lb_get(self, lb):
+        return cmd.LbGetCommand(self, lb)
+
+    def lb_add_health_check(self, lb, vip, **options):
+        return cmd.LbAddHealthCheckCommand(self, lb, vip, **options)
+
+    def lb_del_health_check(self, lb, hc_uuid, if_exists=False):
+        return cmd.LbDelHealthCheckCommand(self, lb, hc_uuid, if_exists)
+
+    def lb_add_ip_port_mapping(self, lb, endport_ip, port_name, source_ip):
+        return cmd.LbAddIpPortMappingCommand(self, lb, endport_ip,
+                                             port_name, source_ip)
+
+    def lb_del_ip_port_mapping(self, lb, endport_ip):
+        return cmd.LbDelIpPortMappingCommand(self, lb, endport_ip)
+
+    def health_check_set_options(self, hc_uuid, **options):
+        return cmd.HealthCheckSetOptionsCommand(self, hc_uuid, **options)
+
+    def health_check_get_options(self, hc_uuid):
+        return cmd.HealthCheckGetOptionsCommand(self, hc_uuid)
+
     def lr_lb_add(self, router, lb, may_exist=False):
         return cmd.LrLbAddCommand(self, router, lb, may_exist)
 
@@ -382,3 +415,20 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
 
     def meter_get(self, meter):
         return cmd.MeterGetCommand(self, meter)
+
+    def bfd_add(self, logical_port, dst_ip, min_tx=None, min_rx=None,
+                detect_mult=None, external_ids=None, options=None,
+                may_exist=False):
+        return cmd.BFDAddCommand(self, logical_port, dst_ip, min_tx=min_tx,
+                                 min_rx=min_rx, detect_mult=detect_mult,
+                                 external_ids=external_ids, options=options,
+                                 may_exist=may_exist)
+
+    def bfd_del(self, uuid):
+        return cmd.BFDDelCommand(self, uuid)
+
+    def bfd_find(self, logical_port, dst_ip):
+        return cmd.BFDFindCommand(self, logical_port, dst_ip)
+
+    def bfd_get(self, uuid):
+        return cmd.BFDGetCommand(self, uuid)
